/* Cairo - a vector graphics library with display and print output
 *
 * Copyright © 2005 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it either under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation
 * (the "LGPL") or, at your option, under the terms of the Mozilla
 * Public License Version 1.1 (the "MPL"). If you do not alter this
 * notice, a recipient may use your version of this file under either
 * the MPL or the LGPL.
 *
 * You should have received a copy of the LGPL along with this library
 * in the file COPYING-LGPL-2.1; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * You should have received a copy of the MPL along with this library
 * in the file COPYING-MPL-1.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY
 * OF ANY KIND, either express or implied. See the LGPL or the MPL for
 * the specific language governing rights and limitations.
 *
 * The Original Code is the cairo graphics library.
 *
 * The Initial Developer of the Original Code is Red Hat, Inc.
 *
 * Contributor(s):
 *	Doodle <doodle@scenergy.dfmk.hu>
 */

/*
 * This file contains the extra API for the OS/2 platform. Currently it
 * contains two extra functions, the cairo_os2_initialize() and the
 * cairo_os2_uninitialize(). Both of them are called automatically if
 * cairo is compiled to a DLL file, but you have to call them before
 * using cairo if you link to cairo statically!
 */

#include "cairoint.h"
#include "fontconfig/fontconfig.h"
#ifdef __WATCOMC__
#include "malloc.h"
#endif

static int bCairoIsInitialized = 0;

HMTX cairo_toy_font_face_hash_table_mutex = NULL;
HMTX cairo_scaled_font_map_mutex = NULL;
HMTX _global_image_glyph_cache_mutex = NULL;
#ifdef CAIRO_HAS_FT_FONT
HMTX cairo_ft_unscaled_font_map_mutex = NULL;
#endif


CAIROEXPORT void
CAIROCALL   cairo_os2_initialize()
{
  /* This may initialize some stuffs, like create mutex semaphores etc.. */

  if (bCairoIsInitialized) return;

  /* Create the mutex semaphores we'll use! */

  /* cairo-font.c: */
  DosCreateMutexSem(NULL, &cairo_toy_font_face_hash_table_mutex, 0, FALSE);
  DosCreateMutexSem(NULL, &cairo_scaled_font_map_mutex, 0, FALSE);
  DosCreateMutexSem(NULL, &_global_image_glyph_cache_mutex, 0, FALSE);

#ifdef CAIRO_HAS_FT_FONT
  /* cairo-ft-font.c: */
  DosCreateMutexSem(NULL, &cairo_ft_unscaled_font_map_mutex, 0, FALSE);
#endif

  /* Initialize FontConfig */
  FcInit();

  bCairoIsInitialized = 1;
}

CAIROEXPORT void
CAIROCALL   cairo_os2_uninitialize()
{
  /* This has to uninitialize some stuffs, like destroy mutex semaphores etc.. */

  if (!bCairoIsInitialized) return;

  /* Free allocated memories! */
  /* (Check cairo_debug_reset_static_date() for an example of this! */
  _cairo_font_reset_static_data();
#ifdef CAIRO_HAS_FT_FONT
  _cairo_ft_font_reset_static_data();
#endif

  /* Destroy the mutex semaphores we've created! */
  /* cairo-font.c: */
  if (cairo_toy_font_face_hash_table_mutex)
  {
    DosCloseMutexSem(cairo_toy_font_face_hash_table_mutex);
    cairo_toy_font_face_hash_table_mutex = NULL;
  }
  if (cairo_scaled_font_map_mutex)
  {
    DosCloseMutexSem(cairo_scaled_font_map_mutex);
    cairo_scaled_font_map_mutex = NULL;
  }
  if (_global_image_glyph_cache_mutex)
  {
    DosCloseMutexSem(_global_image_glyph_cache_mutex);
    _global_image_glyph_cache_mutex = NULL;
  }

#ifdef CAIRO_HAS_FT_FONT
  /* cairo-ft-font.c: */
  if (cairo_ft_unscaled_font_map_mutex)
  {
    DosCloseMutexSem(cairo_ft_unscaled_font_map_mutex);
    cairo_ft_unscaled_font_map_mutex = NULL;
  }
#endif

  bCairoIsInitialized = 0;

  /* Uninitialize FontConfig */
  FcFini();

#ifdef __WATCOMC__
  /* It can happen that the libraries we use have memory leaks,
   * so there are still memory chunks allocated at this point.
   * In these cases, Watcom might still have a bigger memory chunk,
   * called "the heap" allocated from the OS.
   * As we want to minimize the memory we lose from the point of
   * view of the OS, we call this function to shrink that heap
   * as much as possible.
   */
  _heapshrink();
#endif
}
